@@ -1,23 +1,23 @@
 # -*- coding: utf-8 -*-
-"""
-Copyright 2010-2013 Joakim Möller
-Copyright 2014 Marek Rudnicki
 
-This file is part of matlab_wrapper.
+# Copyright 2010-2013 Joakim Möller
+# Copyright 2014 Marek Rudnicki
 
-matlab_wrapper is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+# This file is part of matlab_wrapper.
 
-matlab_wrapper is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+# matlab_wrapper is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+
+# matlab_wrapper is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with matlab_wrapper.  If not, see <http://www.gnu.org/licenses/>.
 
-You should have received a copy of the GNU General Public License
-along with matlab_wrapper.  If not, see <http://www.gnu.org/licenses/>.
-"""
 
 from __future__ import print_function, division, absolute_import
 
@@ -27,6 +27,7 @@ from os.path import join, dirname, isfile, realpath
 import os
 import warnings
 import sys
+import weakref
 
 import ctypes
 from ctypes import c_char_p, POINTER, c_size_t, c_bool, c_void_p, c_int
@@ -128,7 +129,7 @@ class MatlabSession(object):
 
 
         ### Workspace object
-        self.workspace = Workspace(self)
+        self.workspace = Workspace(weakref.ref(self))
 
 
 
@@ -306,22 +307,22 @@ def load_engine_and_libs(matlab_root, options):
 
 
     elif system == 'Darwin':
-	if bits == '64bit':
-	    lib_dir = join(matlab_root, "bin", "maci64")
-	else:
-	    unsopported_paltform(system,bits)
+        if bits == '64bit':
+            lib_dir = join(matlab_root, "bin", "maci64")
+        else:
+            unsopported_paltform(system,bits)
 
-	libeng = Library(
-	    join(lib_dir, 'libeng.dylib')
-	)
-	libmx = Library(
-	    join(lib_dir, 'libmx.dylib')
-	)
+        libeng = Library(
+            join(lib_dir, 'libeng.dylib')
+        )
+        libmx = Library(
+            join(lib_dir, 'libmx.dylib')
+        )
 
-	command = "{executable} {options}".format(
-	    executable=join(matlab_root, 'bin', 'matlab'),
-	    options=options
-	)
+        command = "{executable} {options}".format(
+            executable=join(matlab_root, 'bin', 'matlab'),
+            options=options
+        )
 
 
     else:
@@ -381,13 +382,40 @@ https://github.com/mrkrd/matlab_wrapper""".format(system, bits))
 
 
 class Workspace(object):
-    def __init__(self, session):
-        self._session = session
+    """A convenient interface to MATLAB workspace.
+
+    You can use attributes to access MALTAB functions and variables::
+
+      workspace.sin([1., 2., 3.])
+      pi = workspace.pi()
+      a = workspace.a
+
+
+    In MATLAB the output of the function depends on the number of
+    outputs arguments.  By default, we assume that there is one output
+    argument.  If you would like to change that, add `nout` keyward
+    argument to the function, e.g.::
+
+      sorted,idx = workspace.sort([3,1,2], nout=2)
+
+    """
+    def __init__(self, session_ref):
+        """Workspace constructor.
+
+        Parameters
+        ----------
+        session_ref : weak referecne to MatlabSession
+            We need weak reference here, because Workspace is an
+            attribute of MatlabSession and we do not want cyclic
+            referencing.
+
+        """
+        self._session_ref = session_ref
 
 
     def __getattr__(self, attr):
 
-        session = self._session
+        session = self._session_ref()
 
         session.eval("KIND__ = exist('{}')".format(attr))
         kind = session.get('KIND__')
@@ -400,7 +428,7 @@ class Workspace(object):
             out = session.get(attr)
 
         elif kind in (2, 3, 5, 6): # Function
-            out = MatlabFunction(name=attr, session=self._session)
+            out = MatlabFunction(name=attr, session=session)
 
         else:
             raise NotImplementedError("Unknown variable/function type in MATLAB workspace: {}".format(attr))
@@ -413,7 +441,8 @@ class Workspace(object):
         if name.startswith('_'):
             object.__setattr__(self, name, value)
         else:
-            self._session.put(name, value)
+            session = self._session_ref()
+            session.put(name, value)
 
 
 
@@ -742,6 +771,9 @@ class Library(object):
     `_730` to function names on the fly.  It should resolve the int vs
     mwSize problems for those functions.
 
+    It also initializes library functions by setting `artypes`,
+    `restype` and `errcheck` attributes.
+
     """
     def __init__(self, name, **kwargs):
         self._lib = ctypes.CDLL(name, **kwargs)
